SystemOrganization addCategory: #'Polymorph-Tools-Diff'!

!MCMergeResolutionRequest methodsFor: '*Polymorph-Tools-Diff-override' stamp: 'gvc 2/7/2009 11:26'!
defaultAction
	"Modally open a merge tool."
	
	^self viewMerger! !

!MCMergeResolutionRequest methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 2/3/2009 13:27'!
viewMerger
	"Open a model browser to perform the merge and answer wheter merged."

	^Preferences useNewDiffToolsForMC
		ifTrue: [self viewPatchMerger]
		ifFalse: [(MCMergeBrowser new
					merger: merger;
					label: messageText) showModally]! !

!MCMergeResolutionRequest methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 2/3/2009 13:26'!
viewPatchMerger
	"Open a modal diff tools browser to perform the merge."

	|m modalMorph|
	m := PSMCMergeMorph forMerger: self merger.
	modalMorph := (UIManager default respondsTo: #modalMorph)
		ifTrue: [UIManager default modalMorph]
		ifFalse: [World].
	modalMorph openModal: (
		m newWindow
			title: messageText).
	^m merged! !

!MCAddition methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 2/4/2009 16:35'!
targetClassName
	"Answer the class name of the target since the class may no longer exist."
	
	^self targetClass name! !

ComposableMorph subclass: #DiffMorph
	instanceVariableNames: 'srcText dstText prettyPrint contextClass srcMorph dstMorph scrollbarMorph mapMorph joinMorph difference joinMappings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!DiffMorph class methodsFor: 'as yet unclassified' stamp: 'gvc 10/20/2006 11:26'!
from: old to: new
	"Answer a new instance of the receiver with the given
	old and new text."

	^self new
		from: old
		to: new! !

!DiffMorph class methodsFor: 'as yet unclassified' stamp: 'gvc 9/2/2008 15:46'!
from: old to: new contextClass: aClass
	"Answer a new instance of the receiver with the given
	old and new text."

	^self new
		from: old
		to: new
		contextClass: aClass! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 12:34'!
additionColor
	"Answer the color used to show additions."

	^Color paleGreen alpha: 0.5! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/29/2006 18:23'!
adoptPaneColor: paneColor
	"Change our border color too."
	
	super adoptPaneColor: paneColor.
	paneColor ifNil: [^self].
	self borderStyle baseColor: paneColor! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 10:52'!
applyHighlights
	"Apply the relevant highlights to src and dst."

	self srcMorph highlights: (self joinMappings gather: [:j | j src highlights]).
	self dstMorph highlights: (self joinMappings gather: [:j | j dst highlights])! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 11:11'!
applyJoin
	"Apply the join mappings to the join morph."

	self joinMorph mappings: self joinMappings! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 14:11'!
applyMap
	"Apply the join mappings to the map morph."

	self mapMorph mappings: self joinMappings! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/20/2006 13:11'!
calculateDifference
	"Calculate the difference of the src and dst."

	self difference: ((TextDiffBuilder
		from: self oldText asString to: self newText asString)
			buildPatchSequence)! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 15:56'!
calculateJoinMappings
	"Calculate the join parameters between src and dst
	and store in joinMappings."

	self joinMappings: self calculatedJoinMappings! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 11:51'!
calculatedJoinMappings
	"Calculate the join parameters between src and dst
	and answer.
	sl = src line, dl = dst line, j = joins, ds = dst run start, ss = src run start
	de = dst run end, se = dst run end, mds = match dst start, mss = match src start"

	|sl dl j ds ss de se mds mss|
	sl := dl := 0.
	j := OrderedCollection new.
	ds := de:=  ss := se := mss := mds := 0.
	self difference do: [:p |
		p key = #match ifTrue: [
			sl := sl + 1.
			dl := dl + 1.
			mss = 0 ifTrue: [mss := sl. mds := dl].
			(ds > 0 or: [ss > 0]) ifTrue: [
				ss = 0 ifTrue: [ss := sl].
				ds = 0 ifTrue: [ds := dl].
				se = 0 ifTrue: [se := ss - 1].
				de = 0 ifTrue: [de := ds - 1].
				j add: (self newJoinSectionFrom: (ss to: se) to: (ds to: de)).
				ds := de := ss := se := 0]].
		p key = #remove ifTrue: [
			mss > 0 ifTrue: [
				j add: (self newMatchJoinSectionFrom: (mss to: sl) to: (mds to: dl)).
				mss := mds := 0].
			sl := sl + 1.
			ss = 0 ifTrue: [ss := sl].
			se := sl].
		p key = #insert ifTrue: [
			mss > 0 ifTrue: [
				j add: (self newMatchJoinSectionFrom: (mss to: sl) to: (mds to: dl)).
				mss := mds := 0].
			dl := dl + 1.
			ss > 0 ifTrue: [
				se = 0 ifTrue: [se := ss].
				de = 0 ifTrue: [de := ds].
				j add: (self newJoinSectionFrom: (ss to: se) to: (ds to: de)).
				ds := de := ss := se := 0].
			ds = 0 ifTrue: [ds := dl].
			de := dl]].
	sl := sl + 1.
	dl := dl + 1.
	(ds > 0 or: [ss > 0]) ifTrue: [
		ss = 0 ifTrue: [ss := sl ].	
		ds = 0 ifTrue: [ds := dl].
		se = 0 ifTrue: [se := ss - 1].
		de = 0 ifTrue: [de := ds - 1].
		j add: (self newJoinSectionFrom: (ss to: se) to: (ds to: de))].
	mss > 0 ifTrue: [
		j add: (self newMatchJoinSectionFrom: (mss to: sl - 1) to: (mds to: dl - 1))].
	^j! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/23/2006 16:29'!
calibrateScrollbar
	"Set the scrollbar parameters to match the texts."

	|maxY range delta innerH|
	self fullBounds.
	maxY := self srcMorph textExtent y max: self dstMorph textExtent y.
	innerH := self dstMorph innerBounds height.
	delta := self dstMorph textMorph defaultLineHeight.
	range := maxY - innerH max: 0.
	range = 0 ifTrue: [^self scrollbarMorph scrollDelta: 0.02 pageDelta: 0.2; interval: 1.0; setValue: 0.0].
	self scrollbarMorph
		scrollDelta: (delta / range) asFloat 
		pageDelta: ((innerH - delta) / range) asFloat;
		interval: (innerH / maxY) asFloat;
		setValue: ((self srcMorph scroller offset y max: self dstMorph scroller offset y)
					 / range min: 1.0) asFloat! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 14:29'!
colorForType: type
	"Anwser the color to use for the given change type."

	^{self matchColor. self additionColor. self removalColor. self modificationColor}
		at: (#(match addition removal modification) indexOf: type)! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:38'!
contextClass
	"Answer the value of contextClass"

	^ contextClass! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:38'!
contextClass: anObject
	"Set the value of contextClass"

	contextClass := anObject! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 14:10'!
defaultColor
	"Answer the default color for the receiver."

	^Color white! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 11:04'!
defaultTitle
	"Answer the default title label for the receiver."

	^'Diff' translated! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 11:27'!
difference
	"Answer the value of difference"

	^ difference! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 11:27'!
difference: anObject
	"Set the value of difference"

	difference := anObject! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
dstMorph
	"Answer the value of dstMorph"

	^ dstMorph! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
dstMorph: anObject
	"Set the value of dstMorph"

	dstMorph := anObject! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/25/2006 18:22'!
dstScroll: scrollValue
	"Called from dst when scrolled by keyboard etc."

	self scrollbarMorph value: scrollValue.
	self srcMorph vScrollBarValue: scrollValue.
	self updateJoinOffsets! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:21'!
dstText
	"Answer the value of dstText"

	^ dstText! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:21'!
dstText: anObject
	"Set the value of dstText"

	dstText := anObject! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 12:35'!
edgeColor
	"Answer the color used to show the border of the changes."

	^Color gray alpha: 0.5! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/23/2006 16:27'!
extent: newExtent
	"Update the scrollbar."

	super extent: newExtent.
	self calibrateScrollbar! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 13:23'!
font: aFont
	"Set the font on the src and dst morphs."

	self srcMorph font: aFont.
	self dstMorph font: aFont! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 9/2/2008 15:24'!
from: old to: new
	"Set the old (src) and new (dst) text."

	self
		srcText: old;
		dstText: new.
	self
		setText;
		calculateDifference;
		calculateJoinMappings;
		calibrateScrollbar;
		applyHighlights;
		applyJoin;
		applyMap! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 9/2/2008 15:45'!
from: old to: new contextClass: aClass
	"Set the old (src) and new (dst) text."

	self
		contextClass: aClass;
		srcText: old;
		dstText: new.
	self
		setText;
		calculateDifference;
		calculateJoinMappings;
		calibrateScrollbar;
		applyHighlights;
		applyJoin;
		applyMap! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/23/2006 16:12'!
hideOrShowScrollBar
	"Do nothing"
	! !

!DiffMorph methodsFor: 'initialization' stamp: 'gvc 9/2/2008 16:21'!
initialize
	"Initialize the receiver."

	|exv exh opts ppCheckbox|
	super initialize.
	self prettyPrint: Preferences diffsWithPrettyPrint.
	ppCheckbox := self newPrettyPrintCheckboxMorph.
	opts := self newPanel
		addMorph: ((self newRow: {ppCheckbox}) 
						listCentering: #bottomRight).
	opts vResizing: #shrinkWrap.
	opts extent: opts minExtent.
	self
		srcMorph: self newSrcMorph;
		joinMorph: self newJoinMorph;
		dstMorph: self newDstMorph;
		scrollbarMorph: self newScrollbarMorph;
		mapMorph: self newMapMorph;
		changeProportionalLayout;
		addMorph: self srcMorph fullFrame: (LayoutFrame
			fractions: (0@0 corner: 0.5@1) offsets: (0@0 corner: self joinMorph width negated@opts height negated));
		addMorph: self joinMorph fullFrame: (LayoutFrame
			fractions: (0.5@0 corner: 0.5@1) offsets: (self joinMorph width negated@0 corner: 0@opts height negated));
		addMorph: self dstMorph fullFrame: (LayoutFrame
			fractions: (0.5@0 corner: 1@1) offsets: (0@0 corner: self scrollbarMorph width negated - self mapMorph width@opts height negated));
		addMorph: self scrollbarMorph fullFrame: (LayoutFrame
			fractions: (1@0 corner: 1@1) offsets: (self scrollbarMorph width negated - self mapMorph width@0 corner: self mapMorph width negated@opts height negated));
		addMorph: self mapMorph fullFrame: (LayoutFrame
			fractions: (1@0 corner: 1@1) offsets: (self mapMorph width negated@0 corner: 0@opts height negated));
		addMorph: opts fullFrame: (LayoutFrame
			fractions: (0@1 corner: 1@1) offsets: (0@opts height negated corner: 0@0)).
	exv := ExclusiveWeakMessageSend newSharedState.
	exh := ExclusiveWeakMessageSend newSharedState.
	self srcMorph
		when: #vScroll send: #srcScroll: to: self exclusive: exv;
		when: #hScroll send: #hScrollValue: to: self dstMorph exclusive: exh.
	self dstMorph
		when: #vScroll send: #dstScroll: to: self exclusive: exv;
		when: #hScroll send: #hScrollValue: to: self srcMorph  exclusive: exh.
	self
		linkSubmorphsToSplitters;
		extent: self initialExtent! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/25/2006 17:55'!
join: aJoin selected: aBoolean
	"Set the selection for the given join and update the
		src dst and join morphs."

	aJoin selected: aBoolean.
	self srcMorph changed.
	self joinMorph changed.
	self dstMorph changed! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 12:08'!
joinColor
	"Answer the color used for the join bar."

	^Color paleBlue duller! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/24/2006 15:55'!
joinMappings
	"Answer the join parameters between src and dst."

	^joinMappings ifNil: [self calculateJoinMappings]! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/24/2006 15:55'!
joinMappings: aCollection
	"Set the join parameters between src and dst."

	joinMappings _ aCollection! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
joinMorph
	"Answer the value of joinMorph"

	^ joinMorph! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
joinMorph: anObject
	"Set the value of joinMorph"

	joinMorph := anObject! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:12'!
joinSectionClass
	"Answer the class to use for a new join section."

	^JoinSection! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
mapMorph
	"Answer the value of mapMorph"

	^ mapMorph! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
mapMorph: anObject
	"Set the value of mapMorph"

	mapMorph := anObject! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 14:29'!
matchColor
	"Answer the color used to show matches."

	^Color transparent! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 12:34'!
modificationColor
	"Answer the color used to show changes."

	^Color paleYellow alpha: 0.5! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 13:52'!
newDstMorph
	"Answer a new dst text morph."

	^self newSrcMorph! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/25/2006 12:54'!
newHighlight
	"Anewser a new highlight."

	^TextHighlight new
		color: self modificationColor;
		borderWidth: 1;
		borderColor: self edgeColor! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/25/2006 13:39'!
newHighlight: type
	"Anewser a new highlight."

	^TextHighlight new
		color: (self colorForType: type);
		borderWidth: 1;
		borderColor: self edgeColor;
		fillWidth: true! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:15'!
newJoinMorph
	"Answer a new join morph."

	^DiffJoinMorph new
		hResizing: #shrinkWrap;
		vResizing: #spaceFill;
		extent: 30@4;
		minWidth: 30;
		color: self joinColor! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:11'!
newJoinSection
	"Answer a new join section."

	^self joinSectionClass new
		srcColor: self modificationColor;
		dstColor: self modificationColor;
		borderWidth: 1;
		borderColor: self edgeColor;
		addDependent: self;
		yourself! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 14:08'!
newJoinSectionFrom: srcRange to: dstRange
	"Answer a new join section."

	|spl dpl sy1 sy2 dy1 dy2 t c|
	spl := self srcMorph textMorph paragraph lines.
	dpl := self dstMorph textMorph paragraph lines.
	t := #modification.
	sy1 := srcRange first > spl size
		ifTrue: [t := #addition.
				spl last bottom truncated - 1]
		ifFalse: [(spl at: srcRange first) top truncated - 1].
	sy2 := srcRange size < 1
		ifTrue: [t := #addition.
				 sy1 + 3]
		ifFalse: [srcRange last > spl size
				ifTrue: [spl last bottom truncated + 3]
				ifFalse: [(spl at: srcRange last) bottom truncated - 1]].
	dy1 := dstRange first > dpl size
		ifTrue: [t := #removal.
				dpl last bottom truncated - 1]
		ifFalse: [(dpl at: dstRange first) top truncated - 1].
	dy2 := dstRange size < 1
		ifTrue: [t := #removal.
				dy1 + 3]
		ifFalse: [dstRange last > dpl size
				ifTrue: [dpl last bottom truncated + 3]
				ifFalse: [(dpl at: dstRange last) bottom truncated - 1]].
	c := self colorForType: t.
	^self newJoinSection
		type: t;
		srcColor: c;
		dstColor: c;
		srcLineRange: srcRange;
		dstLineRange: dstRange;
		srcRange: (sy1 to: sy2);
		dstRange: (dy1 to: dy2);
		createHighlightsFrom: self srcMorph textMorph paragraph
		to: self dstMorph textMorph paragraph! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 14:16'!
newMapMorph
	"Answer a new map morph."

	^DiffMapMorph new
		hResizing: #shrinkWrap;
		vResizing: #spaceFill;
		extent: 20@4;
		minWidth: 20;
		borderStyle: (BorderStyle inset width: 1)! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 11:07'!
newMatchJoinSectionFrom: srcRange to: dstRange
	"Answer a new match join section."

	|spl dpl sy1 sy2 dy1 dy2 c|
	spl := self srcMorph textMorph paragraph lines.
	dpl := self dstMorph textMorph paragraph lines.
	sy1 := (spl at: srcRange first) top truncated.
	sy2 := (spl at: srcRange last) bottom truncated.
	dy1 := (dpl at: dstRange first) top truncated.
	dy2 := (dpl at: dstRange last) bottom truncated.
	c := self colorForType: #match.
	^self newJoinSection
		type: #match;
		borderWidth: 0;
		srcColor: c;
		dstColor: c;
		srcLineRange: srcRange;
		dstLineRange: dstRange;
		srcRange: (sy1 to: sy2);
		dstRange: (dy1 to: dy2);
		createHighlightsFrom: self srcMorph textMorph paragraph
		to: self dstMorph textMorph paragraph! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 9/2/2008 15:56'!
newPrettyPrintCheckboxMorph
	"Answer a new checkbox for specifying whether to use
	pretty printing for the diff texts."

	^self
		newCheckboxFor: self
		getSelected: #prettyPrint
		setSelected: #prettyPrint:
		getEnabled: nil
		label: 'Pretty print' translated
		help: 'If selected, pretty print will be applied to any displayed method source (eliminates trivial formatting changes)' translated! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 12/1/2008 11:48'!
newScrollbarMorph
	"Answer a new scrollbar morph."

	^ScrollBar new
		model: self;
		setValueSelector: #vScroll:;
		vResizing: #spaceFill;
		width: self theme scrollbarThickness! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 13:52'!
newSrcMorph
	"Answer a new src text morph."

	^(self newTextEditorFor: nil
		getText: nil
		setText: nil
		getEnabled: nil)
		hideVScrollBarIndefinitely: true;
		borderWidth: 0;
		enabled: false;
		wrapFlag: false;
		selectionColor: self textSelectionColor;
		setText: ''! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/20/2006 11:26'!
newText
	"Answer the new (dst) text."

	^self dstMorph text! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/20/2006 11:26'!
oldText
	"Answer the old (src) text."

	^self srcMorph text! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:21'!
prettyPrint
	"Answer the value of prettyPrint"

	^ prettyPrint! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:22'!
prettyPrint: aBoolean
	"Set the value of prettyPrint"

	prettyPrint == aBoolean ifTrue: [^self].
	prettyPrint := aBoolean.
	self updateText
	! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 12:34'!
removalColor
	"Answer the color used to show removals."

	^Color paleRed alpha: 0.5! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/23/2006 15:47'!
scrollbarMorph
	"Answer the value of scrollbarMorph"

	^ scrollbarMorph! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/23/2006 15:47'!
scrollbarMorph: anObject
	"Set the value of scrollbarMorph"

	scrollbarMorph := anObject! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 12/3/2008 15:06'!
setText
	"Set the src and dst text in the morphs applying
	prettyPrint if required."

	|src dst ctx|
	src := self srcText.
	dst := self dstText.
	ctx := self contextClass.
	(self prettyPrint and: [ctx notNil])
		ifTrue: [src isEmpty ifFalse: [
					src := ctx prettyPrinterClass 
						format: src
						in: ctx
						notifying: nil
						decorated: Preferences colorWhenPrettyPrinting].
				dst isEmpty ifFalse: [
					dst := ctx prettyPrinterClass 
						format: dst
						in: ctx
						notifying: nil
						decorated: Preferences colorWhenPrettyPrinting]].	
	self srcMorph setText: src; font: self theme textFont.
	self dstMorph setText: dst; font: self theme textFont! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
srcMorph
	"Answer the value of srcMorph"

	^ srcMorph! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
srcMorph: anObject
	"Set the value of srcMorph"

	srcMorph := anObject! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/25/2006 18:22'!
srcScroll: scrollValue
	"Called from src when scrolled by keyboard etc.."

	self scrollbarMorph value: scrollValue.
	self dstMorph vScrollBarValue: scrollValue.
	self updateJoinOffsets! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:21'!
srcText
	"Answer the value of srcText"

	^ srcText! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:21'!
srcText: anObject
	"Set the value of srcText"

	srcText := anObject! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 16:11'!
textSelectionColor
	"Answer the color used for thew text selection."

	^Preferences textHighlightColor alpha: 0.5! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 12/1/2008 12:27'!
themeChanged
	"Update the scrollbar width/frame."

	|offset|
	super themeChanged.
	self scrollbarMorph width: self theme scrollbarThickness.
	offset := self scrollbarMorph width negated - self mapMorph width.
	self scrollbarMorph layoutFrame leftOffset: offset.
	self dstMorph layoutFrame rightOffset: offset! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 16:09'!
updateJoinOffsets
	"Update the src and dst offsets in the join morph
	to match the src and dst tex scroll offsets."

	self joinMorph
		srcOffset: 0 @ self srcMorph scroller offset y negated;
		dstOffset: 0 @ self dstMorph scroller offset y negated;
		changed! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 9/2/2008 15:23'!
updateText
	"Reset the text if we have some."

	(self srcText notNil and: [self dstText notNil]) ifTrue: [
		self from: self srcText to: self dstText]! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/25/2006 18:22'!
vScroll: scrollValue
	"Called from standalone scroolbar.
	Scroll the srcMorph and redo the join."

	self srcMorph vScrollBarValue: scrollValue.
	self dstMorph vScrollBarValue: scrollValue.
	self updateJoinOffsets! !

DiffMorph subclass: #MergeDiffMorph
	instanceVariableNames: 'allowJoinClicks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!MergeDiffMorph methodsFor: 'accessing' stamp: 'gvc 1/8/2009 18:17'!
allowJoinClicks
	"Answer the value of allowJoinClicks"

	^ allowJoinClicks! !

!MergeDiffMorph methodsFor: 'accessing' stamp: 'gvc 1/8/2009 18:17'!
allowJoinClicks: anObject
	"Set the value of allowJoinClicks"

	allowJoinClicks := anObject! !

!MergeDiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/8/2009 18:17'!
calculatedJoinMappings
	"Specify click allowance for each section."
	
	^super calculatedJoinMappings do: [:j |
		j allowClick: self allowJoinClicks]! !

!MergeDiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:15'!
compositeText
	"Answer the composite text based on the selection state
	of the joins."

	^self joinMorph compositeText
	! !

!MergeDiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:37'!
indicateDst
	"Change the indicators of the joins to the dst side."

	self joinMappings do: [:section |
		section selectionState: #dst].
	self
		changed;
		changed: #selectedDifferences! !

!MergeDiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:38'!
indicateSrc
	"Change the indicators of the joins to the src side."

	self joinMappings do: [:section |
		section selectionState: #src].
	self
		changed;
		changed: #selectedDifferences! !

!MergeDiffMorph methodsFor: 'initialize-release' stamp: 'gvc 1/8/2009 18:17'!
initialize
	"Initialize the receiver."

	super initialize.
	self
		allowJoinClicks: true! !

!MergeDiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:13'!
joinSectionClass
	"Answer the class to use for a new join section."

	^MergeJoinSection! !

!MergeDiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:15'!
newJoinMorph
	"Answer a new join morph."

	^super newJoinMorph
		when: #joinClicked send: #update: to: self with: #joinClicked! !

!MergeDiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:15'!
update: aspect
	"A join has probably changed its selection state."

	super update: aspect.
	aspect == #joinClicked
		ifTrue: [self
				changed;
				changed: #selectedDifferences]! !

ComposableMorph subclass: #PSMCMergeMorph
	instanceVariableNames: 'patchMorph codeMorph merged'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!PSMCMergeMorph class methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 10:31'!
forMerger: aMerger
	"Answer a new instance of the receiver
	with the given merger as the model."
	
	^ self new model: aMerger! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/9/2009 15:18'!
allConflictsResolved
	"Answer whether all conflicts were resolved."
	
	^(self model ifNil: [^false]) isMerged! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/9/2009 15:43'!
cancel
	"Delete the window to cancel."

	self window delete! !

!PSMCMergeMorph methodsFor: 'accessing' stamp: 'gvc 10/29/2006 18:28'!
codeMorph
	"Answer the value of codeMorph"

	^ codeMorph! !

!PSMCMergeMorph methodsFor: 'accessing' stamp: 'gvc 10/29/2006 18:28'!
codeMorph: anObject
	"Set the value of codeMorph"

	codeMorph := anObject! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 11:53'!
compositeText
	"Answer the composite text from the patch morph."

	^self patchMorph compositeText! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/12/2009 17:12'!
compositeText: aString
	"Save the new method text for the selected method.
	Yet to be implemented."
	
	UIManager default inform: 'Saving of replacement methods is not yet imlemented.'! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 12:52'!
conflictCount
	"Answer the number of conflicts that are unresolved."

	^(self model ifNil: [^0]) conflicts count: [:c | c isResolved not]! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 12:50'!
conflictCountString
	"Answer a string describing the number of conflicts."

	|count|
	count := self conflictCount.
	^count = 1
		ifTrue: ['1 conflict' translated]
		ifFalse: ['{1} conflicts' translated format: {count}]! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 09:47'!
defaultTitle
	"Answer the default title label for the receiver."

	^'Merge' translated! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/9/2009 15:38'!
initialize
	"Initialize the receiver."

	|buttons buttonsHeight|
	super initialize.
	buttons := self newButtonsMorph.
	buttonsHeight := buttons minExtent y.
	self
		merged: false;
		patchMorph: self newPatchMorph;
		codeMorph: self newCodeMorph;
		changeProportionalLayout;
		addMorph: self patchMorph
		fullFrame: (LayoutFrame fractions: (0@0 corner: 1@0.6));
		addMorph: self codeMorph
		fullFrame: (LayoutFrame fractions: (0@0.6 corner: 1@1) offsets: (0@0 corner: 0@buttonsHeight negated));
		addMorph: self newButtonsMorph
		fullFrame: (LayoutFrame fractions: (0@1 corner: 1@1) offsets: (0@buttonsHeight negated corner: 0@0));
		addPaneSplitters! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/9/2009 15:44'!
merge
	"Do the merge and close if no conflicts."
	
	^self model isMerged
		ifTrue: [self merged: true.
				self window delete]
		ifFalse: [self inform: 'You must resolve all conflicts first.' translated]! !

!PSMCMergeMorph methodsFor: 'accessing' stamp: 'gvc 1/8/2009 17:42'!
merged
	"Answer the value of merged. Indicates whether
	the merge button was pressed with no conflicts remaining."

	^ merged! !

!PSMCMergeMorph methodsFor: 'accessing' stamp: 'gvc 1/8/2009 17:42'!
merged: anObject
	"Set the value of merged"

	merged := anObject! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 11:11'!
model: aMerger
	"Set the model and update the window."

	super model: aMerger.
	self patchMorph
		model: (aMerger operations, aMerger conflicts)! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 12:43'!
newButtonsMorph
	"Answer a new buttons morph."

	^(self newRow: {
			self newConflictsButton.
			self newToolSpacer hResizing: #spaceFill.
			self newMergeButton.
			self newCancelButton})
		removeProperty: #fillStyle;
		listCentering: #bottomRight;
		layoutInset: 4! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 11:53'!
newCodeMorph
	"Answer a new code morph."

	^(self newTextEditorFor: self
		getText: #compositeText
		setText: #compositeText:
		getEnabled: nil)
		wrapFlag: false;
		setText: ''! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 12:54'!
newConflictsButton
	"Answer a new button for displaying the count
	of outstanding conflicts and navigating to each.
	It will be disabled if all conflicts are resolved."

	^(self
		newButtonFor: self
		getState: nil
		action:  #selectNextConflict
		arguments: #()
		getEnabled: #notAllConflictsResolved
		getLabel:  #conflictCountString
		help: 'Select the next conflict in the tree' translated)
		vResizing: #spaceFill "workaround until table  layout fixed"! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 12:42'!
newMergeButton
	"Answer a new button for performing the merge.
	It will be disabled if any conflicts are unresolved."

	^self
		newButtonFor: self
		action:  #merge
		getEnabled:  #allConflictsResolved
		label:  'Merge' translated
		help: 'Merge the version into the image' translated! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:26'!
newPatchMorph
	"Answer a new patch morph."

	^PSMCMergePatchMorph new
		borderWidth: 0;
		addDependent: self;
		yourself! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 12:46'!
notAllConflictsResolved
	"Answer whether any conflicts are unresolved."
	
	^self allConflictsResolved not! !

!PSMCMergeMorph methodsFor: 'accessing' stamp: 'gvc 10/29/2006 18:28'!
patchMorph
	"Answer the value of patchMorph"

	^ patchMorph! !

!PSMCMergeMorph methodsFor: 'accessing' stamp: 'gvc 10/29/2006 18:28'!
patchMorph: anObject
	"Set the value of patchMorph"

	patchMorph := anObject! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 12:57'!
selectNextConflict
	"Select the next conflict in the tree."
	
	self patchMorph selectNextConflict! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 13:22'!
update: aspect
	"A join has probably changed its selection state."

	super update: aspect.
	aspect == #compositeText
		ifTrue: [self updateCode].
	aspect == #changes
		ifTrue: [self
				changed: #conflictCountString;
				changed: #allConflictsResolved;
				changed: #notAllConflictsResolved]! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 11:54'!
updateCode
	"Update the code morph to match selected differences."

	self changed: #compositeText! !

ComposableMorph subclass: #PSMCPatchMorph
	instanceVariableNames: 'diffMorph changeTree selectedChangeWrapper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

PSMCPatchMorph subclass: #PSMCMergePatchMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 14:28'!
changesMenu: m
	"Answer the changes menu."

	|menu|
	menu := super changesMenu: m.
	menu addLine.
	menu
		addToggle: 'Keep current version' translated
		target: self
		selector: #keepCurrentVersion
		getStateSelector: nil
		enablementSelector: #selectionIsConflict.
	menu lastItem
	 	font: self theme menuFont;
		icon: MenuIcons smallBackIcon.	
	menu
		addToggle: 'Use incoming version' translated
		target: self
		selector: #useIncomingVersion
		getStateSelector: nil
		enablementSelector: #selectionIsConflict.
	menu lastItem
	 	font: self theme menuFont;
		icon: MenuIcons smallForwardIcon.
	menu
		addToggle: 'Mark as conflict' translated
		target: self
		selector: #markAsConflict
		getStateSelector: nil
		enablementSelector: #selectionIsConflict.
	menu lastItem
	 	font: self theme menuFont;
		icon: MenuIcons smallCancelIcon.
	menu addLine.
	menu
		addToggle: 'Select next conflict' translated
		target: self
		selector: #selectNextConflict
		getStateSelector: nil
		enablementSelector: #notAllConflictsResolved.
	menu lastItem
	 	font: self theme menuFont;
		icon: MenuIcons smallRightFlushIcon.	
	^menu! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:25'!
diffMorphClass
	"Answer a the class to use for a new diff morph."

	^MergeDiffMorph! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:46'!
keepCurrentVersion
	"Mark the conflict as local."
	
	self selectedChangeWrapper chooseLocal.
	self changed: #changes.
	self updateSource! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:45'!
markAsConflict
	"Mark the conflict as unresolved."
	
	self selectedChangeWrapper clearChoice.
	self changed: #changes.
	self updateSource.! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 13:45'!
nextConflict
	"Answer the first conflict or, if the current selection is a conflict,
	the subsequent conflict."

	|coll current index|
	current := self selectedChangeWrapper.
	coll := current isConflict
		ifTrue: [(self model
					copyFrom: (index := self model indexOf: current item) + 1
					to: self model size),
				(self model copyFrom: 1 to: index)]
		ifFalse: [self model].
	^coll detect: [:item | item isConflict] ifNone: [nil]! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 14:26'!
notAllConflictsResolved
	"Answer whether any conflicts are unresolved."
	
	^self model anySatisfy: [:item | item isConflict and: [item isResolved not]]! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 3/9/2009 14:21'!
selectNextConflict
	"Select the next conflict after the current selection, if any."

	|next def|
	next := self nextConflict ifNil: [^self].
	def := next remoteDefinition ifNil: [next localDefinition].
	self changed: {#changes. #openPath. next targetClassName asString. def selector asString}! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 14:40'!
selectionIsConflict
	"Answer whether the currently selected change is a conflict."

	^self selectedChangeWrapper isNil
		ifTrue: [false]
		ifFalse: [self selectedChangeWrapper isConflict]! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/12/2009 14:06'!
updateSource
	"Update the source difference morph."

	|sel|
	sel := self selectedChangeWrapper.
	self diffMorph allowJoinClicks: (sel notNil and: [
		sel isConflict and: [sel operation isModification]]).
	super updateSource.
	(sel isNil or: [sel isConflict not]) ifTrue: [^self].
	sel localChosen
		ifTrue: [self diffMorph indicateSrc]
		ifFalse: [self diffMorph indicateDst]! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:46'!
useIncomingVersion
	"Mark the conflict as remote."
	
	self selectedChangeWrapper chooseRemote.
	self changed: #changes.
	self updateSource! !

!PSMCPatchMorph class methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 11:08'!
forPatch: aPatch
	"Answer a new instance of the receiver
	with the given patch as the model."
	
	^ self new model: aPatch operations! !

!PSMCPatchMorph class methodsFor: 'as yet unclassified' stamp: 'gvc 2/3/2009 13:12'!
initialize
	"Set up extra diff preferences here."

	Preferences
		addPreference: #useNewDiffToolsForMC
		categories: #(browsing)
		default: true
		balloonHelp: 'When enabled the Polymorph diff tools will be used with Monticello. When diabled, the original tools are used.' translated! !

!PSMCPatchMorph class methodsFor: 'as yet unclassified' stamp: 'gvc 4/17/2007 17:51'!
taskbarIcon
	"Answer the icon for the receiver in a task bar."

	^MenuIcons smallForwardIcon! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/14/2009 12:45'!
browseClass
	"Browse the class of the selected item."

	ToolSet default
		browse: self selectedChangeWrapper actualClass
		selector: self selectedMessageName! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/14/2009 12:47'!
browseImplementors
	"Browse the method implementors."

	self systemNavigation
		browseAllImplementorsOf: (self selectedMessageName ifNil: [^self])! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/14/2009 12:47'!
browseSenders
	"Browse the method senders."

	self systemNavigation
		browseAllCallsOn: (self selectedMessageName ifNil: [^self])! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/14/2009 12:47'!
browseVersions
	"Browse the method versions."

	ToolSet default
		browseVersionsOf: self selectedChangeWrapper actualClass
		selector: (self selectedMessageName ifNil: [^self])! !

!PSMCPatchMorph methodsFor: 'accessing' stamp: 'gvc 1/29/2009 13:17'!
changeTree
	"Answer the value of changeTree"

	^ changeTree! !

!PSMCPatchMorph methodsFor: 'accessing' stamp: 'gvc 1/29/2009 13:17'!
changeTree: anObject
	"Set the value of changeTree"

	changeTree := anObject! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 11:41'!
changes
	"Answer the changes tree roots."

	|changes classes|
	self model ifNil: [^#()].
	changes := SortedCollection sortBlock: [:a :b | a asString <= b asString].
	classes := Set new.
	self model do: [:o |
		o definition isOrganizationDefinition
			ifTrue: [changes add: (o patchWrapper model: self model)]
			ifFalse: [(classes includes: o targetClassName)
					ifFalse: [classes add: o targetClassName.
							changes add: (PSMCClassChangeWrapper with: o targetClassName model: self model)]]].
	^changes! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/14/2009 12:35'!
changesMenu: m
	"Answer the changes menu."

	|menu|
	menu := self newMenu
		addTitle: 'Changes'
		icon: MenuIcons smallCopyIcon.		
	menu
		addToggle: 'Browse class...' translated
		target: self
		selector: #browseClass
		getStateSelector: nil
		enablementSelector: nil.
	menu lastItem
	 	font: self theme menuFont;
		icon: Browser taskbarIcon.
	menu addLine.
	menu
		addToggle: 'Versions...' translated
		target: self
		selector: #browseVersions
		getStateSelector: nil
		enablementSelector: #selectionIsMethodChange.
	menu lastItem
	 	font: self theme menuFont;
		icon: MenuIcons smallJustifiedIcon.	
	menu
		addToggle: 'Senders...' translated
		target: self
		selector: #browseSenders
		getStateSelector: nil
		enablementSelector: #selectionIsMethodChange.
	menu lastItem
	 	font: self theme menuFont;
		icon: MenuIcons smallForwardIcon.	
	menu
		addToggle: 'Implementors...' translated
		target: self
		selector: #browseImplementors
		getStateSelector: nil
		enablementSelector: #selectionIsMethodChange.
	menu lastItem
	 	font: self theme menuFont;
		icon: MenuIcons smallDoItIcon.
	^menu! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 12:02'!
compositeText
	"Answer the composite text based on the selection state
	of the joins."

	^self diffMorph compositeText! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/29/2006 18:18'!
defaultTitle
	"Answer the default title label for the receiver."

	^'Changes' translated! !

!PSMCPatchMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 11:56'!
diffMorph
	"Answer the value of diffMorph"

	^ diffMorph! !

!PSMCPatchMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 11:56'!
diffMorph: anObject
	"Set the value of diffMorph"

	diffMorph := anObject! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:25'!
diffMorphClass
	"Answer a the class to use for a new diff morph."

	^DiffMorph! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 13:16'!
initialize
	"Initialize the receiver."

	super initialize.
	self diffMorph: self newDiffMorph.
	self changeTree: self newChangeTreeMorph.
	self
		changeProportionalLayout;
		addMorph: self changeTree
		fullFrame: (LayoutFrame fractions: (0@0 corner: 0.3@1));
		addMorph: self diffMorph
		fullFrame: (LayoutFrame fractions: (0.3@0 corner: 1@1)
					offsets: (ProportionalSplitterMorph splitterWidth @ 0 corner: 0@0));
		addPaneSplitters! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 11:08'!
model: aCollection
	"Set the model and update the window."

	super model: aCollection.
	self changed: #changes! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 15:33'!
newChangeTreeMorph
	"Answer a new morph for the tree of changes."

	^(self
		newTreeFor: self
		list: #changes
		selected: #selectedChangeWrapper
		changeSelected: #selectedChangeWrapper:)
		getMenuSelector: #changesMenu:! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:25'!
newDiffMorph
	"Answer a new morph for the source difference."

	^self diffMorphClass new
		borderStyle: (BorderStyle inset width: 1);
		font: self theme statusFont;
		addDependent: self;
		yourself! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 15:45'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then
perform it on myself. If not, send it to otherTarget, presumably the
editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ otherTarget perform: selector]! !

!PSMCPatchMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 11:58'!
selectedChange
	"Answer the selected change."

	^(self selectedChangeWrapper ifNil: [^nil]) operation! !

!PSMCPatchMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 11:57'!
selectedChangeWrapper
	"Answer the selected change."

	^selectedChangeWrapper! !

!PSMCPatchMorph methodsFor: 'accessing' stamp: 'gvc 11/1/2006 12:18'!
selectedChangeWrapper: aWrapper
	"Set the selected change."

	selectedChangeWrapper := aWrapper.
	self
		changed: #selectedChangeWrapper;
		updateSource;
		changed: #compositeText! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 15:52'!
selectedMessageName
	"Answer the method selector or nil if no method change
	is selected.."

	^self selectionIsMethodChange
		ifTrue: [self selectedChange definition selector]
		! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 15:40'!
selectionIsMethodChange
	"Answer whether the currently selected change is for a method."

	^self selectedChange isNil
		ifTrue: [false]
		ifFalse: [self selectedChange definition isMethodDefinition]! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 15:30'!
update: aspect
	"A join has probably changed its selection state."

	super update: aspect.
	aspect == #selectedDifferences
		ifTrue: [self changed: #compositeText]! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/14/2009 13:28'!
updateSource
	"Update the source difference morph."

	|sel|
	sel := self selectedChange.
	sel isNil
		ifTrue: [self diffMorph
				from: ''
				to: '']
		ifFalse: [self diffMorph
					from: sel diffFromSource
					to: sel diffToSource
					contextClass: (sel isClassPatch ifTrue: [nil] ifFalse: [sel targetClass])]! !

!MCConflict methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 10/30/2006 11:27'!
patchWrapper
	"Answer a wrapper for a patch tree for the receiver."

	^PSMCConflictWrapper with: self! !

!MCConflict methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 10/30/2006 11:30'!
targetClassName
	"Answer the class name of the target since the class may no longer exist."
	
	^operation targetClassName! !

ListItemWrapper subclass: #PSMCChangeWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!PSMCChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 1/14/2009 12:42'!
actualClass
	"Answer the class represented in the receiver."

	^(self operation ifNil: [^nil]) targetClass! !

!PSMCChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 11:23'!
icon
	"Answer a form with an icon to represent the receiver"
	|o|
	o := self operation.
	o isNil ifTrue: [^MenuIcons smallJumpIcon].
	o isAddition ifTrue: [^MenuIcons smallOkIcon].
	o isRemoval ifTrue: [^MenuIcons smallCancelIcon].
	^MenuIcons smallForwardIcon! !

!PSMCChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 14:41'!
isConflict
	"Answer whether the receiver is a conflict item."

	^false! !

!PSMCChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 11:16'!
model: anObject
	"Set the model."

	model _ anObject! !

!PSMCChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 11:22'!
operation
	"Answer the underlying operation."

	self subclassResponsibility ! !

PSMCChangeWrapper subclass: #PSMCClassChangeWrapper
	instanceVariableNames: 'conflict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!PSMCClassChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 1/14/2009 12:44'!
actualClass
	"Answer the class represented in the receiver."

	^super actualClass ifNil: [Smalltalk classNamed: self item]! !

!PSMCClassChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 14:44'!
chooseLocal
	"Choose the local version."

	self conflict chooseLocal! !

!PSMCClassChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 14:44'!
chooseRemote
	"Choose the remote version."

	self conflict chooseRemote! !

!PSMCClassChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 14:44'!
clearChoice
	"Choose neither version (be in conflict)."

	self conflict clearChoice! !

!PSMCClassChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:44'!
conflict
	"Answer the conflict for the class itself or nil if none."

	|o|
	conflict ifNotNil: [^conflict].
	o := self model detect: [:i | i targetClassName == self item and: [
				i definition isClassDefinition]] ifNone: [].
	o ifNil: [^nil].
	o isConflict ifTrue: [conflict _ o].
	^conflict! !

!PSMCClassChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 1/14/2009 12:13'!
contents
	"Answer the contents of the change."

	^(self model select: [:i | i targetClassName = self item and: [
			i definition isClassDefinition not]])
		collect: [:o |  o patchWrapper model: self model]! !

!PSMCClassChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:11'!
icon
	"Answer a form with an icon to represent the receiver"
	
	self conflict ifNotNilDo: [:c |
		c localChosen ifTrue: [^MenuIcons smallBackIcon]].
	^super icon! !

!PSMCClassChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 14:41'!
isConflict
	"Answer whether the receiver is a conflict item."

	^self conflict notNil! !

!PSMCClassChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:43'!
localChosen
	"Answer whether the local version is chosen."

	^self conflict localChosen! !

!PSMCClassChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 1/14/2009 12:43'!
operation
	"Answer the patch operation for the class itself or nil if none."

	|o|
	o := self model detect: [:i | i targetClassName = self item and: [
				i definition isClassDefinition]] ifNone: [].
	o ifNil: [^nil].
	o isConflict ifTrue: [^o operation].
	^o! !

!PSMCClassChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 14:39'!
preferredColor
	"Answer the colour for the string. 
	If a conflict and unresolved answer red."
	
	^(self conflict
			ifNil: [true]
			ifNotNilDo: [:c | c isResolved])
		ifTrue: [super preferredColor]
		ifFalse: [Color red]! !

!PSMCClassChangeWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:42'!
remoteChosen
	"Answer whether the remote version is chosen."

	^self conflict remoteChosen! !

PSMCChangeWrapper subclass: #PSMCConflictWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!PSMCConflictWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 11:24'!
asString
	"Answer the method name."
	
	^self operation shortSummary! !

!PSMCConflictWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 14:42'!
chooseLocal
	"Choose the local version."

	self item chooseLocal! !

!PSMCConflictWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 14:43'!
chooseRemote
	"Choose the remote version."

	self item chooseRemote! !

!PSMCConflictWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 14:43'!
clearChoice
	"Choose neither version (be in conflict)."

	self item clearChoice! !

!PSMCConflictWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:09'!
icon
	"Answer a form with an icon to represent the receiver"
	
	self item localChosen ifTrue: [^MenuIcons smallBackIcon].
	^super icon! !

!PSMCConflictWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 14:42'!
isConflict
	"Answer whether the receiver is a conflict item."

	^true! !

!PSMCConflictWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:41'!
localChosen
	"Answer whether the local version is chosen."

	^self item localChosen! !

!PSMCConflictWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 11:34'!
operation
	"Answer the patch operation for the conflict."

	^self item operation! !

!PSMCConflictWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 11:43'!
preferredColor

	^self item isResolved
		ifTrue: [super preferredColor]
		ifFalse: [Color red]! !

!PSMCConflictWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:41'!
remoteChosen
	"Answer whether the remote version is chosen."

	^self item remoteChosen! !

PSMCChangeWrapper subclass: #PSMCPatchOperationWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

PSMCPatchOperationWrapper subclass: #PSMCOrganizationChangeWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!PSMCPatchOperationWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 11:44'!
asString
	"Answer the method name."
	
	^self item shortSummary! !

!PSMCPatchOperationWrapper methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 12:00'!
operation
	"Answer the pacth operation for the receiver or nil if none."

	^self item! !

TextDiffBuilder subclass: #InlineTextDiffBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!InlineTextDiffBuilder methodsFor: 'as yet unclassified' stamp: 'gvc 9/2/2008 16:10'!
split: aString
	"Answer the split 'lines' by splitting on whitespace."
	
	|str lines sep|
	lines := OrderedCollection new.
	sep := Character separators, '^()[]{}''"`;.'.
	str := aString readStream.
	[str atEnd] whileFalse: [
		lines add: (str upToAny: sep).
		str atEnd ifFalse: [
			str skip: -1.
			lines add: str next asString]].
	^lines! !

!MCVersionHistoryBrowser methodsFor: '*Polymorph-Tools-Diff-override' stamp: 'gvc 2/3/2009 13:15'!
viewChanges
	"Note that the patchLabel will be parsed in MCPatchBrowser>>installSelection, so don't translate it!!"
	
	self viewChanges: (self baseSnapshot patchRelativeToBase: self selectedSnapshot)
	! !

!MCVersionHistoryBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 2/3/2009 13:15'!
viewChanges: patch
	"Opene a browser on the patch."
	
	|patchLabel|
	patchLabel := 'Changes between {1} and {2}' format: { self selectedInfo name. ancestry name }.
	Preferences useNewDiffToolsForMC
		ifTrue: [((PSMCPatchMorph forPatch: patch) newWindow)
					title: patchLabel;
					open]
		ifFalse: [(MCPatchBrowser forPatch: patch)
					label: patchLabel;
					show]! !

ProportionalSplitterMorph subclass: #DiffJoinMorph
	instanceVariableNames: 'srcOffset dstOffset mappings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!DiffJoinMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 12:06'!
compositeText
	"Answer the composite text based on the selection state
	of the joins."

	|t|
	t _ Text new.
	self mappings do: [:j |
		j appendToCompositeText: t].
	^t! !

!DiffJoinMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/20/2006 14:21'!
defaultColor
	"Answer the default color for the receiver."

	^Color transparent! !

!DiffJoinMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 12:08'!
drawOn: aCanvas
	"Draw the indicators for the mappings."
	
	super drawOn: aCanvas.
	aCanvas translateBy: self topLeft clippingTo: self clippingBounds during: [:c |
		self mappings do: [:j | j drawOn: c]]! !

!DiffJoinMorph methodsFor: 'accessing' stamp: 'gvc 10/24/2006 16:02'!
dstOffset
	"Answer the value of dstOffset"

	^ dstOffset! !

!DiffJoinMorph methodsFor: 'accessing' stamp: 'gvc 10/31/2006 11:33'!
dstOffset: anInteger
	"Set the dstOffset."

	dstOffset := anInteger.
	self mappings do: [:j |
		j dstOffset: anInteger]! !

!DiffJoinMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 13:18'!
extent: aPoint
	"Update the shapes of the joins."
	
	super extent: aPoint.
	self updateMappings! !

!DiffJoinMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 11:44'!
initialize
	"Initialize the receiver."

	super initialize.
	self
		mappings: OrderedCollection new;
		srcOffset: 0@0;
		dstOffset: 0@0! !

!DiffJoinMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 13:18'!
layoutBounds: aRectangle
	"Set the bounds for laying out children of the receiver."
	
	super layoutBounds: aRectangle.
	self updateMappings! !

!DiffJoinMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 11:50'!
mappings
	"Answer the value of mappings"

	^ mappings! !

!DiffJoinMorph methodsFor: 'accessing' stamp: 'gvc 10/31/2006 13:19'!
mappings: anObject
	"Set the value of mappings"

	mappings := anObject.
	self updateMappings.
	self changed! !

!DiffJoinMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 12:00'!
mouseDown: evt
	"Check for a click."

	|cj|
	cj := self mappings
		detect: [:j | j containsPoint: evt position - self topLeft]
		ifNone: [].
	cj ifNotNil: [
		cj clicked.
		self triggerEvent: #joinClicked].
	super mouseDown: evt! !

!DiffJoinMorph methodsFor: 'accessing' stamp: 'gvc 10/24/2006 16:02'!
srcOffset
	"Answer the value of srcOffset"

	^ srcOffset! !

!DiffJoinMorph methodsFor: 'accessing' stamp: 'gvc 10/31/2006 11:33'!
srcOffset: anInteger
	"Set the srcOffset."

	srcOffset := anInteger.
	self mappings do: [:j |
		j srcOffset: anInteger]! !

!DiffJoinMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 13:18'!
updateMappings
	"Update the shapes of the joins."
	
	self mappings do: [:j |
		j width: self width]! !

!MCDefinition methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 12/12/2008 16:18'!
actualClass
	"Since the targetClass call on a patch operation will fail
	otherwise."
	
	^nil! !

BorderedMorph subclass: #DiffMapMorph
	instanceVariableNames: 'mappings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!DiffMapMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 15:11'!
adoptPaneColor: paneColor
	"Change our border color too."
	
	|c|
	super adoptPaneColor: paneColor.
	paneColor ifNil: [^self].
	c := paneColor alphaMixed: 0.1 with: Color white.
	self fillStyle: ((GradientFillStyle
			ramp: (self gradientRampForColor: c))
		origin: self bounds topLeft;
		direction: 0@ self height).
	self borderStyle baseColor: paneColor! !

!DiffMapMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 14:10'!
defaultColor
	"Answer the default color for the receiver."

	^Color white! !

!DiffMapMorph methodsFor: 'nil' stamp: 'gvc 10/26/2006 15:05'!
drawOn: aCanvas
	"Draw the indicators for the mappings."
	
	|b f|
	b := self innerBounds insetBy: 2.
	super drawOn: aCanvas.
	b height < 1 ifTrue: [^self].
	f := self mappingsHeight.
	f < 1 ifTrue: [^self].
	f := b height / f.
	aCanvas clipBy: self clippingBounds during: [:c |
		self mappings do: [:j |
			j drawMapOn: c in: b scale: f]]! !

!DiffMapMorph methodsFor: 'accessing' stamp: 'gvc 3/21/2008 17:14'!
extent: newExtent
	"Update the gradient."
	
	super extent: newExtent.
	(self fillStyle notNil and: [self fillStyle isOrientedFill])
		ifTrue: [self fillStyle direction: 0@self height]! !

!DiffMapMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 15:12'!
gradientRampForColor: c
	"Answer the background gradient ramp to use."
	
	^{0.0->c darker duller. 0.1-> c lighter.
		0.9->c twiceLighter. 1.0->c darker}! !

!DiffMapMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 14:08'!
initialize
	"Initialize the receiver."

	super initialize.
	self
		mappings: #()! !

!DiffMapMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 13:57'!
mappings
	"Answer the value of mappings"

	^ mappings! !

!DiffMapMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 14:15'!
mappings: anObject
	"Set the value of mappings"

	mappings := anObject.
	self changed! !

!DiffMapMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 10:51'!
mappingsHeight
	"Answer the maximum y of all the mappings."
	
	self mappings ifEmpty: [^0].
	^self mappings last dst range last ! !

!MCPatchBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 10/24/2006 15:44'!
diffSelection
	"Open a diff browser on the selection."
	
	selection ifNotNil:
		[selection diff]! !

!MCPatchBrowser methodsFor: '*Polymorph-Tools-Diff-override' stamp: 'gvc 3/9/2009 14:30'!
methodListMenu: aMenu

	selection ifNotNil:
		[aMenu addList:#(
			('install'	 installSelection)
			('diff'	diffSelection)
			-)].
	super methodListMenu: aMenu.
	^ aMenu
! !

!MCModification methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 9/2/2008 15:52'!
diff
	"Open a diff browser on the changes."

	(DiffMorph
		from: self fromSource
		to: self toSource
		contextClass: (self isClassPatch ifTrue: [nil] ifFalse: [self targetClass]))
			extent: 400@300;
			openInWindow! !

!MCModification methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 1/14/2009 14:11'!
diffFromSource
	"Answer fromSource of the modification. If a class patch then answer
	the fromSource with the class-side definition and comment appended."
	
	^self isClassPatch
		ifTrue: [self fromSource, String cr, String cr,
				obsoletion classDefinitionString, String cr, String cr,
				obsoletion commentStamp, String cr,
				obsoletion comment]
		ifFalse: [self fromSource]! !

!MCModification methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 1/14/2009 14:12'!
diffToSource
	"Answer toSource of the modification. If a class patch then answer
	the toSource with the class-side definition and comment appended."
	
	^self isClassPatch
		ifTrue: [self toSource, String cr, String cr, 
				modification classDefinitionString, String cr, String cr,
				modification commentStamp, String cr,
				modification comment]
		ifFalse: [self toSource]! !

!MCModification methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 1/14/2009 12:10'!
targetClassName
	"Answer the class name of the target since the class may no longer exist."
	
	^self targetClass name! !

!MCRemoval methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 10/30/2006 10:41'!
targetClassName
	"Answer the class name of the target since the class may no longer exist."
	
	^definition className! !

!MCPatchOperation methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 10/24/2006 15:40'!
diff
	"Open a diff browser on the changes."
! !

!MCPatchOperation methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 1/14/2009 14:07'!
diffFromSource
	"Answer fromSource of the operation for a diff tool."
	
	^self fromSource! !

!MCPatchOperation methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 1/14/2009 14:07'!
diffToSource
	"Answer toSource of the operation for a diff tool."
	
	^self toSource! !

!MCPatchOperation methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 10/30/2006 11:15'!
patchWrapper
	"Answer a wrapper for a patch tree for the receiver."

	^PSMCPatchOperationWrapper with: self! !

!MCPatchOperation methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 1/14/2009 14:05'!
shortSummary
	"Answer a short summary of the receiver."

	|suffix|
	suffix := self fromSource = self toSource
		ifTrue: [' (revision changed)']
		ifFalse: [''].
	^(self isClassPatch
		ifTrue: [self definition summary]
		ifFalse: [self definition isOrganizationDefinition
				ifTrue: [self definition description last]
				ifFalse: [self definition selector asString]]), suffix! !

!MCPatchOperation methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 10/30/2006 10:40'!
targetClassName
	"Answer the class name of the target since the class may no longer exist."
	
	self subclassResponsibility! !

!VersionsBrowser methodsFor: '*Polymorph-Tools-Diff-override' stamp: 'gvc 2/9/2009 12:45'!
compareToOtherVersion
	"Prompt the user for a reference version, then spawn a window 
	showing the diffs between the older and the newer of the current 
	version and the reference version as text."

	| change1 change2 s1 s2 |
	change1 := changeList at: listIndex ifAbsent: [^self].
	change2 := UIManager default
				chooseFrom: (list copyWithoutIndex: listIndex)
				values: (changeList copyWithoutIndex: listIndex).
	 change2 ifNil: [^self].
	s1 := change1 string.
	s2 := change2 string.
	s1 = s2 ifTrue: [^self inform: 'Exact Match' translated].
	(DiffMorph
		from: s1
		to: s2
		contextClass: change1 methodClass)
		openInWindowLabeled: (('Comparison from {1} to {2}' translated) format: {change1 stamp. change2 stamp})! !

!MCVersionInspector methodsFor: '*Polymorph-Tools-Diff-override' stamp: 'gvc 2/3/2009 13:23'!
changes
	"Open a patch morph for the changes."
	
	self viewChanges: (self version package snapshot patchRelativeToBase: self version snapshot)! !

!MCVersionInspector methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 2/3/2009 13:18'!
viewChanges: patch
	"Open a patch morph for the changes."
	
	|title|
	title := 'Changes from {1}' format: {self version info name}.
	Preferences useNewDiffToolsForMC
		ifTrue: [((PSMCPatchMorph forPatch: patch) newWindow)
					title: title;
					open]
		ifFalse: [(MCPatchBrowser forPatch: self version changes)
					showLabelled: title]! !

!ChangeList methodsFor: '*Polymorph-Tools-Diff-override' stamp: 'gvc 9/2/2008 15:47'!
compareToCurrentVersion
	"If the current selection corresponds to a method in the system, then spawn a window showing the diffs as text"

	| change class s1 s2 |
	listIndex = 0
		ifTrue: [^ self].
	change := changeList at: listIndex.
	((class := change methodClass) notNil
			and: [class includesSelector: change methodSelector])
		ifTrue: [s1 := (class sourceCodeAt: change methodSelector) asString.
			s2 := change string.
			s1 = s2
				ifTrue: [^ self inform: 'Exact Match'].
			(DiffMorph
				from: s2
				to: s1
				contextClass: class)
				openInWindowLabeled: 'Comparison to Current Version']
		ifFalse: [self flash]! !

Object subclass: #JoinSection
	instanceVariableNames: 'src dst borderWidth borderColor type width shape'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:16'!
addHighlightsFrom: srcBlock to: dstBlock to: aCollection color: aColor
	"Add the highlights required for the given character blocks
	of a paragraph. May be up to three highlights depending
	on the line spans."
	
	srcBlock textLine = dstBlock textLine
		ifTrue: [aCollection add: (TextHighlight new
				color: aColor;
				bounds: (srcBlock topLeft corner: dstBlock bottomRight))]
		ifFalse: [aCollection
					add: (TextHighlight new
							color: aColor;
							bounds: (srcBlock topLeft corner: srcBlock textLine bottomRight));
					add: (TextHighlight new
							fillWidth: true;
							color: aColor;
							bounds: (srcBlock bottomLeft corner: dstBlock topRight));
					add: (TextHighlight new
							color: aColor;
							bounds: (dstBlock textLine topLeft corner: dstBlock bottomRight))]! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:16'!
borderColor
	"Answer the value of borderColor"

	^ borderColor! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:16'!
borderColor: aColor
	"Set the value of borderColor"

	borderColor := aColor.
	self updateHighlights! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:22'!
borderColorToUse
	"Answer the border color to use."
	
	^self borderColor! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:16'!
borderWidth
	"Answer the value of borderWidth"

	^ borderWidth! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:16'!
borderWidth: anInteger
	"Set the value of borderWidth"

	borderWidth := anInteger.
	self src highlight notNil ifTrue: [
		self src highlight borderWidth: anInteger].
	self dst highlight notNil ifTrue: [
		self dst highlight borderWidth: anInteger]! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:17'!
clicked
	"The receiver or a highlight was clicked."

	self wantsClick ifFalse: [^false].
	^true! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:18'!
containsPoint: aPoint
	"Answer whether the receiver contains the given point."

	^self shape containsPoint: aPoint! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:18'!
createHighlights
	"Create and store the src and dst highlights."

	|s d|
	s := OrderedCollection new.
	d := OrderedCollection new.
	s add: (self newHighlight
			color: self src color;
			borderWidth: self borderWidth;
			bounds: (0@self src range first corner: 0@(self src range last + 1));
			borderSides: #(top left bottom)).
	d add: (self newHighlight
			color: self dst color;
			borderWidth: self borderWidth;
			bounds: (0@self dst range first corner: 0@(self dst range last + 1));
			borderSides: #(top right bottom)).
	self src highlights: s.
	self dst highlights: d! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:18'!
createHighlightsFrom: srcPara to: dstPara
	"Create and store the src and dst highlights.
	Use the given paragraphs to determine inline
	diffs."

	|s d si di  srcText dstText diffs i sb eb line|
	self createHighlights.
	self src lineRange notEmpty
		ifTrue: [ line := srcPara lines at: self src lineRange first.
				si := line first.
				line := srcPara lines at: self src lineRange last.
				srcText := srcPara string copyFrom: si to: line last]
		ifFalse: [srcText := ''].
	self dst lineRange notEmpty
		ifTrue: [line := dstPara lines at: self dst lineRange first.
				di := line first.
				line := dstPara lines at: self dst lineRange last.
				dstText := dstPara string copyFrom: di to: line last]
		ifFalse: [dstText := ''].
	self src text: srcText.
	self dst text: dstText.
	self type = #modification ifFalse: [^self].
	s := self src highlights.
	d := self dst highlights.
	diffs := (InlineTextDiffBuilder from: srcText to: dstText)
		buildPatchSequence
			aggregateRuns: [:e | e key].
	diffs do: [:c |
		c first key = #match
			ifTrue: [c do: [:a |
					si := si + a value size.
					di := di + a value size]].
		c first key = #insert
			ifTrue: [i := di.
					c do: [:a | di := di + a value size].
					sb := dstPara characterBlockForIndex: i.
					eb := dstPara characterBlockForIndex: di - 1.
					self addHighlightsFrom: sb to: eb to: d color: (Color green alpha: 0.3)].
		c first key = #remove
			ifTrue: [i := si.
					c do: [:a | si := si + a value size].
					sb := srcPara characterBlockForIndex: i.
					eb := srcPara characterBlockForIndex: si - 1.
					self addHighlightsFrom: sb to: eb to: s color: (Color red alpha: 0.3)]]! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:24'!
drawMapOn: aCanvas in: rect scale: scale
	"Draw the join on the given canvas scaled into the given rectangle."
	
	self type = #match ifTrue: [^self].
	aCanvas
		frameAndFillRectangle: (rect left @ (((self dst range first max: 0) * scale) truncated + rect top)
						corner: (rect right @ ((self dst range last * scale) truncated + rect top)))
		fillColor: (self fillStyleFor: rect)
		borderWidth: 1
		borderColor: self borderColorToUse! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:23'!
drawOn: aCanvas
	"Draw the join on the given canvas."
	
	|v bc|
	(self src color isTransparent and: [self dst color isTransparent])
		ifTrue: [^self].
	v := self shape vertices.
	aCanvas
		drawPolygon: v
		fillStyle: (self fillStyleFor: self shape bounds).
	(self borderWidth > 0 and: [self borderColor isTransparent not]) ifTrue: [
		bc := self borderColorToUse.
		aCanvas
			line: v first + (0@self borderWidth // 2)
			to: v second + (-1@self borderWidth // 2)
			width: self borderWidth
			color: bc;
			line: v third - (1@(self borderWidth // 2))
			to: v fourth - (0@(self borderWidth // 2))
			width: self borderWidth
			color: bc]! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
dst
	"Answer the value of dst"

	^ dst! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
dst: anObject
	"Set the value of dst"

	dst := anObject! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
dstColor: aColor
	"Set the dst color"

	self dst color: aColor! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
dstLineRange: anInterval
	"Set the dst lineRange."

	self dst lineRange: anInterval! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
dstOffset: aPoint
	"Set the dst offset."

	self dst offset:  aPoint.
	self updateShape! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
dstRange: anInterval
	"Set the dst range."

	self dst range: anInterval.
	self updateShape! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:19'!
fillStyleFor: rect
	"Answer the fillStyle to use for the given rectangle."
	
	^self src color = self dst color
		ifTrue: [self src color]
		ifFalse: [(GradientFillStyle ramp: {0.0 -> self src color. 1.0 -> self dst color})
				direction: rect width@0;
				origin: rect topLeft]! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:20'!
initialize
	"Initialize the receiver."

	super initialize.
	self
		src: JoinSide new;
		dst: JoinSide new;
		shape: Polygon new;
		width: 0;
		borderWidth: 0;
		borderColor: Color transparent;
		type: #modification! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:18'!
newHighlight
	"Anwser a new highlight."

	^TextHighlight new
		borderWidth: 1;
		borderColor: self borderColor;
		fillWidth: true! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
shape
	"Answer the value of shape"

	^ shape! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
shape: anObject
	"Set the value of shape"

	shape := anObject! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
src
	"Answer the value of src"

	^ src! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
src: anObject
	"Set the value of src"

	src := anObject! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
srcColor: aColor
	"Set the src color."

	self src color: aColor! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
srcLineRange: anInterval
	"Set the src lneRange."

	self src lineRange: anInterval! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
srcOffset: aPoint
	"Set the src offset"

	self src offset:  aPoint.
	self updateShape! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
srcRange: anInterval
	"Set the  src range"

	self src range: anInterval.
	self updateShape! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
type
	"Answer the value of type"

	^ type! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
type: anObject
	"Set the value of type"

	type := anObject! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:28'!
updateHighlights
	"Update the highlight border colors."

	|bc|
	(self src isNil or: [self dst isNil]) ifTrue: [^self].
	bc := self borderColorToUse.
	self src highlight notNil ifTrue: [
		self src highlight borderColor: bc].
	self dst highlight notNil ifTrue: [
		self dst highlight borderColor: bc]! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:19'!
updateShape
	"Update the receiver's shape."

	(self src range isNil or: [self dst range isNil]) ifTrue: [^self].
	self shape: (Polygon vertices:
		{(0@ self src range first) + self src offset. (self width @ self dst range first) + self dst offset.
		(self width @ self dst range last) + self dst offset. (0@self src range last) + self src offset})! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:19'!
wantsClick
	"Don't if we are transparent for now."

	^(self src color isTransparent and: [self dst color isTransparent]) not! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:21'!
width
	"Answer the value of width"

	^ width! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:21'!
width: anObject
	"Set the value of width"

	width := anObject.
	self updateShape! !

JoinSection subclass: #MergeJoinSection
	instanceVariableNames: 'selectedBorderColor selected selectionState stateIcons allowClick'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 1/8/2009 18:04'!
allowClick
	"Answer the value of allowClick"

	^ allowClick! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 1/8/2009 18:04'!
allowClick: anObject
	"Set the value of allowClick"

	allowClick := anObject! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 12:10'!
appendToCompositeText: aText
	"If appropriate append the relevant src or dst text to the given text."
	
	self selectionState == #src ifTrue: [^aText append: self src text].
	self selectionState == #dst ifTrue: [^aText append: self dst text].
	self selectionState == #both ifTrue: [
		^aText
			append: self src text;
			append: self dst text]! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:22'!
borderColorToUse
	"Answer the border color to use based on the selection state."
	
	^self selected
		ifTrue: [self selectedBorderColor]
		ifFalse: [super borderColorToUse]! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 13:09'!
clicked
	"The receiver or a highlight was clicked."

	self wantsClick ifFalse: [^false].
	self selectNextState.
	^true! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:18'!
containsPoint: aPoint
	"Answer whether the receiver contains the given point."

	^(super containsPoint: aPoint) or: [
		self stateIcon notNil and: [self stateIconBounds containsPoint: aPoint]]! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 13:05'!
defaultStateIcons
	"Answer the default state icons."

	^{MenuIcons smallBackIcon.
		MenuIcons smallForwardIcon.
		MenuIcons smallOkIcon.
		MenuIcons smallCancelIcon}! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:19'!
drawOn: aCanvas
	"Draw the join on the given canvas."
	
	super drawOn: aCanvas.
	self stateIcon ifNotNilDo: [:i |
		aCanvas
			translucentImage: i
			at: self stateIconBounds topLeft]! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 1/8/2009 18:05'!
initialize
	"Initialize the receiver."

	self
		allowClick: true;
		selected: false;
		selectionState: #dst;
		stateIcons: self defaultStateIcons;
		selectedBorderColor: Color black.
	super initialize! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 13:03'!
selectNextState
	"Set the selection state to the next one with wraparound."

	self selectionState: (
		self selectionStates at: (
			(self selectionStates indexOf: self selectionState) \\
				self selectionStates size + 1))! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/25/2006 17:47'!
selected
	"Answer the value of selected"

	^ selected! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/30/2006 15:25'!
selected: aBoolean
	"Set the value of selected"

	selected := aBoolean.
	self
		updateHighlights;
		changed: #selected! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/25/2006 17:43'!
selectedBorderColor
	"Answer the value of selectedBorderColor"

	^ selectedBorderColor! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/25/2006 17:43'!
selectedBorderColor: anObject
	"Set the value of selectedBorderColor"

	selectedBorderColor := anObject! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/31/2006 13:00'!
selectionState
	"Answer the value of selectionState"

	^ selectionState! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/31/2006 13:00'!
selectionState: anObject
	"Set the value of selectionState"

	selectionState := anObject! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 13:01'!
selectionStates
	"Answer the valid selection states in order."

	^#(src dst both neither)! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:14'!
stateIcon
	"Answer the state icon to use."

	self wantsClick ifFalse: [^nil].
	^self stateIcons at: (self selectionStates indexOf: self selectionState)! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 13:28'!
stateIconBounds
	"Answer the bounds of the state icon."
	
	|i|
	i _ self stateIcon ifNil: [^nil].
	^self shape bounds center - (i extent // 2)
		extent: i extent! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/31/2006 13:03'!
stateIcons
	"Answer the value of stateIcons"

	^ stateIcons! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/31/2006 13:03'!
stateIcons: anObject
	"Set the value of stateIcons"

	stateIcons := anObject! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 1/8/2009 18:04'!
wantsClick
	"Allow if explictly enabled and super."

	^self allowClick and: [super wantsClick]! !

Object subclass: #JoinSide
	instanceVariableNames: 'range offset lineRange highlights color text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
color
	"Answer the value of color"

	^ color! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
color: anObject
	"Set the value of color"

	color := anObject! !

!JoinSide methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 10:58'!
highlight
	"Answer the primary highlight."

	^(self highlights ifEmpty: [^nil]) first! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
highlights
	"Answer the value of highlights"

	^ highlights! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
highlights: anObject
	"Set the value of highlights"

	highlights := anObject! !

!JoinSide methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 11:09'!
initialize
	"Initialize the receiver."

	super initialize.
	self
		highlights: #();
		offset: 0@0;
		range: (1 to: 1);
		lineRange: (1 to: 0);
		color: Color yellow;
		text: ''! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
lineRange
	"Answer the value of lineRange"

	^ lineRange! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
lineRange: anObject
	"Set the value of lineRange"

	lineRange := anObject! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
offset
	"Answer the value of offset"

	^ offset! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
offset: anObject
	"Set the value of offset"

	offset := anObject! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
range
	"Answer the value of range"

	^ range! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
range: anObject
	"Set the value of range"

	range := anObject! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 11:09'!
text
	"Answer the value of text"

	^ text! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 11:09'!
text: anObject
	"Set the value of text"

	text := anObject! !

!MCWorkingCopyBrowser methodsFor: '*Polymorph-Tools-Diff-override' stamp: 'gvc 2/3/2009 13:02'!
viewChanges
	| patch |
	'Finding changes' displayProgressAt: Sensor cursorPoint from: 0 to: 10 during:[:bar|
		self canSave ifTrue:[
		bar value: 1.
		patch := workingCopy changesRelativeToRepository: self repository].
		patch isNil ifTrue: [^ self].
		bar value:3.
		patch isEmpty
			ifTrue: [ workingCopy modified: false.
				bar value: 10.
				self inform: 'No changes' ]
			ifFalse:
				[ workingCopy modified: true.
				bar value: 5.
				self viewChanges: patch]]! !

!MCWorkingCopyBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 2/3/2009 13:11'!
viewChanges: patch
	"Open a browser on the given patch."

	Preferences useNewDiffToolsForMC
		ifTrue: [((PSMCPatchMorph forPatch: patch) newWindow)
					title: 'Changes to ', workingCopy description;
					open]
		ifFalse: [(MCPatchBrowser forPatch: patch)
					label: 'Patch Browser: ', workingCopy description;
					show]! !

!MCClassDefinition methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 1/14/2009 13:41'!
classDefinitionString
	"Answer a string describing the class-side definition."
	
	^String streamContents: [:stream | self printClassDefinitionOn: stream]! !

!MCClassDefinition methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 1/14/2009 13:41'!
printClassDefinitionOn: stream
	"Print a class-side definition of the receiver on the given stream.
	Class instance variables and class traits."
	
		stream
			nextPutAll: self className;
			nextPutAll: ' class';
			cr; tab.
		self hasClassTraitComposition ifTrue: [
			stream 
				nextPutAll: 'uses: ';
		 		nextPutAll: self classTraitCompositionString;
				cr; tab ].
		stream
			nextPutAll: 'instanceVariableNames: ';
			store: self classInstanceVariablesString! !

!MCOrganizationDefinition methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 10/30/2006 11:18'!
patchWrapper
	"Answer a wrapper for a patch tree for the receiver."

	^PSMCOrganizationChangeWrapper with: self! !
PSMCPatchMorph initialize!
